/*
 * Copyright 2004-2005 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.ws.muws.v1_0;

import javax.xml.namespace.QName;

/**
 * Constants for names defined by the MUWS 1.0 specifications.
 */
public interface MuwsConstants
{

    /**
     * The base URI for all URIs defined by the WSDM specs.
     */
    String BASE_URI_WSDM = "http://docs.oasis-open.org/wsdm/2004/12/";

    /**
     * The base URI for all URIs defined by the MUWS specs.
     */
    String BASE_URI_MUWS = BASE_URI_WSDM + "muws/";

    /**
     * The base URI for all manageability capability URIs defined by the MUWS specs.
     */
    String BASE_URI_MUWS_CAPABILITIES = BASE_URI_MUWS + "capabilities/";

    /**
     * Namespace URI for MUWS Part 1 schema.
     */
    String NSURI_MUWS_PART1_SCHEMA = BASE_URI_MUWS + "wsdm-muws-part1.xsd";

    /**
     * Namespace prefix for MUWS Part 1 schema.
     */
    String NSPREFIX_MUWS_PART1_SCHEMA = "muws-p1-xs";

    /**
     * Namespace URI for Properties Boolean Match (PBM) schema.
     */
    String NSURI_PBM = BASE_URI_MUWS + "wsdm-pbm.xsd";

    /**
     * Namespace prefix for Properties Boolean Match (PBM) schema.
     */
    String NSPREFIX_PBM = "pbm";

    /**
     * Properties Boolean Match (PBM) CorrelatableProperties expression dialect.
     */
    String PBM_DIALECT = BASE_URI_WSDM + "pbm";

    /**
     * Namespace URI for MUWS Part 2 schema.
     */
    String NSURI_MUWS_PART2_SCHEMA = BASE_URI_MUWS + "wsdm-muws-part2.xsd";

    /**
     * Namespace prefix for MUWS Part 2 schema.
     */
    String NSPREFIX_MUWS_PART2_SCHEMA = "muws-p2-xs";

    /**
     * Namespace URI for MUWS Part 2 WSDL.
     */
    String NSURI_MUWS_PART2_WSDL = BASE_URI_MUWS + "wsdm-muws-part2.wsdl";

    /**
     * Namespace prefix for MUWS Part 2 WSDL.
     */
    String NSPREFIX_MUWS_PART2_WSDL = "muws-p2-wsdl";

    /**
     * Namespace URI for MUWS Part 2 topics.
     */
    String NSURI_MUWS_PART2_TOPICS = BASE_URI_MUWS + "wsdm-muws-part2-events.xml";

    /**
     * Namespace prefix for MUWS Part 2 topics.
     */
    String NSPREFIX_MUWS_PART2_TOPICS = "muws-p2-events";

    /**
     * Situation QNames
     */
    QName SITUATION_AVAILABILITY = new QName(NSURI_MUWS_PART2_SCHEMA, "AvailabilitySituation" ,NSPREFIX_MUWS_PART2_SCHEMA);
    QName SITUATION_CAPABILITY = new QName(NSURI_MUWS_PART2_SCHEMA, "CapabilitySituation" ,NSPREFIX_MUWS_PART2_SCHEMA);
    QName SITUATION_CONFIGURATION = new QName(NSURI_MUWS_PART2_SCHEMA, "ConfigurationSituation" ,NSPREFIX_MUWS_PART2_SCHEMA);
    QName SITUATION_STOP = new QName(NSURI_MUWS_PART2_SCHEMA, "StopSituation" ,NSPREFIX_MUWS_PART2_SCHEMA);
    QName SITUATION_START = new QName(NSURI_MUWS_PART2_SCHEMA, "StartSituation" ,NSPREFIX_MUWS_PART2_SCHEMA);
    QName SITUATION_REQUEST = new QName(NSURI_MUWS_PART2_SCHEMA, "RequestSituation" ,NSPREFIX_MUWS_PART2_SCHEMA);
    QName SITUATION_DESTROY = new QName(NSURI_MUWS_PART2_SCHEMA, "DestroySituation" ,NSPREFIX_MUWS_PART2_SCHEMA);
    QName SITUATION_CREATE = new QName(NSURI_MUWS_PART2_SCHEMA, "CreateSituation" ,NSPREFIX_MUWS_PART2_SCHEMA);
    QName SITUATION_DEPENDENCY = new QName(NSURI_MUWS_PART2_SCHEMA, "DependencySituation" ,NSPREFIX_MUWS_PART2_SCHEMA);
    QName SITUATION_CONNECT = new QName(NSURI_MUWS_PART2_SCHEMA, "ConnectSituation" ,NSPREFIX_MUWS_PART2_SCHEMA);
    QName SITUATION_REPORT = new QName(NSURI_MUWS_PART2_SCHEMA, "ReportSituation" ,NSPREFIX_MUWS_PART2_SCHEMA);
    QName SITUATION_OTHER = new QName(NSURI_MUWS_PART2_SCHEMA, "OtherSituation" ,NSPREFIX_MUWS_PART2_SCHEMA);

}
