/*=============================================================================*
 *  Copyright 2005 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muws.v1_0.capability.impl;

import org.apache.ws.muws.v1_0.capability.RelationshipsCapability;
import org.apache.ws.resource.ResourceContext;
import org.apache.ws.resource.properties.NamespaceVersionHolder;
import org.apache.ws.resource.properties.ResourceProperty;
import org.apache.ws.resource.properties.ResourcePropertySet;
import org.apache.ws.resource.properties.impl.AbstractResourcePropertiesPortType;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.xmlbeans.XmlObject;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.QueryRelationshipsByTypeDocument;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.QueryRelationshipsByTypeResponseDocument;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipType;

import javax.xml.namespace.QName;
import javax.xml.rpc.JAXRPCException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * TODO
 */
public class RelationshipsCapabilityImpl extends AbstractResourcePropertiesPortType implements RelationshipsCapability
{

    /**
     * @param resourceContext
     */
    public RelationshipsCapabilityImpl( ResourceContext resourceContext )
    {
        super( resourceContext );
    }

    /**
     * @param requestDoc
     *
     * @return QueryRelationshipsByTypeResponse document
     */
    public QueryRelationshipsByTypeResponseDocument queryRelationshipsByType(
            QueryRelationshipsByTypeDocument requestDoc )
    {
        QueryRelationshipsByTypeResponseDocument responseDoc = createQueryRelationshipsByTypeResponseDocument();
        QueryRelationshipsByTypeResponseDocument.QueryRelationshipsByTypeResponse response = responseDoc.getQueryRelationshipsByTypeResponse();

        org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.QueryRelationshipsByTypeDocument.QueryRelationshipsByType queryRelationshipsByType = requestDoc.getQueryRelationshipsByType();
        javax.xml.namespace.QName requestedType = queryRelationshipsByType.getRequestedType();

        ResourcePropertySet properties = getProperties();
        ResourceProperty resourceProperty = properties.get(RelationshipsCapability.PROP_NAME_RELATIONSHIP);

        if(resourceProperty == null)
        {
            throw new JAXRPCException("Unable to query relationship since the Resource does not expose a " + RelationshipsCapability.PROP_NAME_RELATIONSHIP + " resource property.");
        }

        List matchingRelationships = new ArrayList();

        Iterator iterator = resourceProperty.iterator();
        while (iterator.hasNext())
        {
            org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipType relationship = (org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipType) iterator.next();
            org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipTypeType type = relationship.getType();
            XmlObject[] childElements = XmlBeanUtils.getChildElements(type);

            for (int i = 0; i < childElements.length; i++)
            {
                XmlObject childElement = childElements[i];
                QName typeName = XmlBeanUtils.getName(childElement);

                if (typeName.equals(requestedType))
                {
                    matchingRelationships.add(relationship);
                }
            }
        }

        //add matching relationships to the response
        response.setRelationshipArray((RelationshipType[])matchingRelationships.toArray(new RelationshipType[0]));

        return responseDoc;
    }

    protected NamespaceVersionHolder getNamespaceSet()
    {
        return null;  // not currently used - will be needed once we need to support multiple spec versions
    }

    private QueryRelationshipsByTypeResponseDocument createQueryRelationshipsByTypeResponseDocument()
    {
        QueryRelationshipsByTypeResponseDocument responseDoc =
                QueryRelationshipsByTypeResponseDocument.Factory.newInstance();
        responseDoc.addNewQueryRelationshipsByTypeResponse();
        return responseDoc;
    }

}
