package org.apache.ws.muws.v1_0.events.impl;

import org.apache.ws.XmlObjectWrapper;
import org.apache.ws.muws.Category;
import org.apache.ws.muws.v1_0.events.Situation;
import org.apache.ws.muws.v1_0.events.SubstitutionalMsg;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.xmlbeans.XmlObject;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.SituationDocument;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.SituationType;

import java.util.Calendar;


/**
 * @author Sal Campana
 */
public class SituationImpl implements Situation, XmlObjectWrapper
{
    private short m_priority = -1;
    private short m_severity = -1;
    private String m_message;
    private Calendar m_situationTime;
    private Boolean m_successDisposition;
    private SubstitutionalMsg m_substitutionMsg;
    private Category m_situationCategory;

    public SituationImpl(Category situationCategory)
    {
        if (situationCategory == null)
        {
            throw new IllegalArgumentException("Category may not be null.");
        }
        m_situationCategory = situationCategory;
    }

    public void setPriority(short priority)
    {
        if (priority < 0 || priority > 100)
        {
            throw new IllegalArgumentException("priority may only be a value from 0 to 100.");
        }
        m_priority = priority;
    }

    public void setSeverity(short severity)
    {
        if (severity < 0 || severity > 6)
        {
            throw new IllegalArgumentException("Severity may only be a value from 0 to 6.");
        }
        m_severity = severity;
    }

    public void setMessage(String message)
    {

        m_message = message;
    }

    public void setSituationTime(Calendar cal)
    {
        m_situationTime = cal;
    }

    public void setSuccessDisposition(Boolean bool)
    {
        m_successDisposition = bool;
    }

    public void setSubstitutionalMsg(SubstitutionalMsg msg)
    {
        m_substitutionMsg = msg;
    }

    public Category getSituationCategory()
    {
        return m_situationCategory;
    }

    public short getPriority()
    {
        return m_priority;
    }

    public short getSeverity()
    {
        return m_severity;
    }

    public String getMessage()
    {
        return m_message;
    }

    public Calendar getSituationTime()
    {
        return m_situationTime;
    }

    public Boolean getSuccessDisposition()
    {
        return m_successDisposition;
    }

    public SubstitutionalMsg getSubstitutionalMsg()
    {
        return m_substitutionMsg;
    }

    public XmlObject getXmlObject()
    {
        SituationDocument situationDocument = SituationDocument.Factory.newInstance();

        org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.SituationType situationType = situationDocument.addNewSituation();
        org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.SituationCategoryType situationCategoryType = situationType.addNewSituationCategory();
        XmlBeanUtils.addChildElement(situationCategoryType, ((XmlObjectWrapper) m_situationCategory).getXmlObject());

        if (m_message != null)
        {
            org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.LangString langString = situationType.addNewMessage();
            langString.setLang(m_message);
        }

        if (m_substitutionMsg != null)
        {
            situationType.setSubstitutableMsg((org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.SubstitutableMsgType) ((XmlObjectWrapper)m_substitutionMsg).getXmlObject());
            org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.SubstitutableMsgType substitutableMsgType = situationType.addNewSubstitutableMsg();
        }

        if (m_priority != -1)
        {
            situationType.setPriority(m_priority);
        }

        if (m_severity != -1)
        {
            situationType.setSeverity(m_severity);
        }


        if (m_situationTime != null)
        {
            situationType.setSituationTime(m_situationTime);
        }

        if (m_successDisposition != null)
        {
            if (m_successDisposition.booleanValue() == true)
            {
                situationType.setSuccessDisposition(SituationType.SuccessDisposition.SUCCESSFUL);
            }
            else
            {
                situationType.setSuccessDisposition(SituationType.SuccessDisposition.UNSUCCESSFUL);
            }
        }
        return situationDocument;
    }
}
