package org.apache.ws.muws.v1_0.events.impl;

import org.apache.ws.XmlObjectWrapper;
import org.apache.ws.muws.v1_0.events.ManagementEvent;
import org.apache.ws.muws.v1_0.events.Situation;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.ws.util.uuid.UuidGeneratorFactory;
import org.apache.xmlbeans.XmlObject;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ComponentAddressType;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ManagementEventDocument;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;


/**
 * @author Sal Campana
 */
public class XmlBeansManagementEvent implements XmlObjectWrapper, ManagementEvent
{
    private String m_reporterResourceId;
    private String m_sourceResourceId;
    private String m_eventId;
    private Object[] m_reporterComponentAddresses;
    private Object[] m_sourceComponentAddresses;
    private Object[] m_any;
    private Situation m_situation;

    public XmlBeansManagementEvent(Situation situation)
    {
        if(situation == null)
        {
            throw new IllegalArgumentException("Situation may not be null.");
        }
        m_situation = situation;
    }

    protected XmlObject buildManagementEventNotifDocXmlBean()
    {
        ManagementEventDocument managementEventDocument = ManagementEventDocument.Factory.newInstance();
        org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ManagementEventType managementEventType = managementEventDocument.addNewManagementEvent();

        org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ComponentType reporterComponentType = managementEventType.addNewReporterComponent();
        if (m_reporterResourceId != null)
        {
            reporterComponentType.setResourceId(m_reporterResourceId);
        }
        if (m_reporterComponentAddresses != null)
        {
            List reporterComponentAddresses = new ArrayList();

            for (int i = 0; i < m_reporterComponentAddresses.length; i++)
            {
                ComponentAddressType reporterComponentAddressType = reporterComponentType.addNewComponentAddress();
                XmlBeanUtils.addChildElement(reporterComponentAddressType, (XmlObject) m_reporterComponentAddresses[i]);
                reporterComponentAddresses.add(reporterComponentAddressType);
            }

            reporterComponentType.setComponentAddressArray((ComponentAddressType[]) reporterComponentAddresses.toArray(new ComponentAddressType[0]));
        }

        org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ComponentType sourceComponentType = managementEventType.addNewSourceComponent();
        if (m_sourceResourceId != null)
        {
            sourceComponentType.setResourceId(m_sourceResourceId);
        }

        if (m_sourceComponentAddresses != null)
        {
            List sourceComponentAddresses = new ArrayList();
            for (int i = 0; i < m_sourceComponentAddresses.length; i++)
            {
                ComponentAddressType sourceComponentAddressType = sourceComponentType.addNewComponentAddress();
                XmlBeanUtils.addChildElement(sourceComponentAddressType, (XmlObject) m_sourceComponentAddresses[i]);
                sourceComponentAddresses.add(sourceComponentAddressType);
            }

            sourceComponentType.setComponentAddressArray((ComponentAddressType[]) sourceComponentAddresses.toArray(new ComponentAddressType[0]));
        }

        if (m_eventId != null)
        {
            managementEventType.setEventId(m_eventId);
        }
        else
        {
            managementEventType.setEventId(UuidGeneratorFactory.createUUIdGenerator().generateUuid());
        }

        managementEventType.setReportTime(Calendar.getInstance());

        if (m_any != null)
        {
            for (int i = 0; i < m_any.length; i++)
            {
                XmlBeanUtils.addChildElement(managementEventType, (XmlObject) m_any[i] );
            }
        }

        XmlBeanUtils.addChildElement(managementEventType, ((XmlObjectWrapper)m_situation).getXmlObject());
        return managementEventDocument;
    }

    public XmlObject getXmlObject()
    {
        return buildManagementEventNotifDocXmlBean();
    }

    public String getReporterResourceId()
    {
        return m_reporterResourceId;
    }

    public String getSourceResourceId()
    {
        return m_sourceResourceId;
    }

    public String getEventId()
    {
        return m_eventId;
    }

    public Object[] getReporterComponentAddresses()
    {
        return m_reporterComponentAddresses;
    }

    public Object[] getSourceComponentAddresses()
    {
        return m_sourceComponentAddresses;
    }

    public Object[] getAny()
    {
        return m_any;
    }

    public void setReporterResourceId(String reporterResourceId)
    {
        m_reporterResourceId = reporterResourceId;
    }

    public void setSourceResourceId(String sourceResourceId)
    {
        m_sourceResourceId = sourceResourceId;
    }

    public void setEventId(String eventId)
    {
        m_eventId = eventId;
    }

    public void setReporterComponentAddresses(Object[] reporterComponentAddresses)
    {
        m_reporterComponentAddresses = reporterComponentAddresses;
    }

    public void setSourceComponentAddresses(Object[] sourceComponentAddresses)
    {
        m_sourceComponentAddresses = sourceComponentAddresses;
    }

    public void setAny(Object[] any)
    {
        m_any = any;
    }

    public Situation getSituation()
    {
        return m_situation;
    }
}
